@@ -54,7 +54,6 @@ class DBusInterface(Component, dbus.service.Object):
     @dbus.service.method("de.pycage.mediabox.control", in_signature = "u")
     def seek(self, pos):
     
-        print "SEEK", pos
         self.emit_message(msgs.MEDIA_ACT_SEEK, pos / 1000.0)
 
 
@@ -79,6 +78,26 @@ class DBusInterface(Component, dbus.service.Object):
 
     @dbus.service.signal("de.pycage.mediabox.control", "uu")
     def seek_signal(self, pos, total): pass
+
+
+    @dbus.service.signal("de.pycage.mediabox.control", "ssss")
+    def load_signal(self, name, info, resource, mimetype): pass
+
+
+    @dbus.service.signal("de.pycage.mediabox.control", "ss")
+    def tag_signal(self, key, value): pass
+    
+    
+    def handle_MEDIA_EV_LOADED(self, player, f):
+    
+        self.load_signal(f.name, f.info, f.resource, f.mimetype)
+        self.__need_report_position = True
+    
+    
+    def handle_MEDIA_EV_TAG(self, key, value):
+    
+        print key, value, type(key), type(value)
+        self.tag_signal(key, value or "")
     
     
     def handle_MEDIA_EV_PLAY(self):
