@@ -8,9 +8,10 @@ def get_devices():
     from AudioArtistStorage import AudioArtistStorage
     from AudioGenreStorage import AudioGenreStorage
     from ImageStorage import ImageStorage
+    from CameraStorage import CameraStorage
     
     from HistoryDevice import HistoryDevice
-    from BookmarkDevice import GenericBookmarkDevice
+    #from BookmarkDevice import GenericBookmarkDevice
     #from BookmarkDevice import AudioBookmarkDevice
     #from BookmarkDevice import VideoBookmarkDevice
     #from BookmarkDevice import ImageBookmarkDevice
@@ -20,9 +21,10 @@ def get_devices():
                AudioArtistStorage,
                AudioGenreStorage,
                ImageStorage,
+               CameraStorage,
 
-               HistoryDevice,
-               GenericBookmarkDevice]
+               HistoryDevice]
+               #GenericBookmarkDevice]
                #AudioBookmarkDevice,
                #VideoBookmarkDevice,
                #ImageBookmarkDevice]
