@@ -3,6 +3,9 @@ from theme import theme
 
 
 class GenericThumbnailer(Thumbnailer):
+    """
+    Thumbnailer for generic non-media files and folders.
+    """
 
     def __init__(self):
     
@@ -18,7 +21,7 @@ class GenericThumbnailer(Thumbnailer):
     def make_quick_thumbnail(self, f):
 
         if (f.mimetype == "application/x-folder"):    
-            return (theme.mb_filetype_folder.get_path(), True)
+            return (theme.mb_folder_normal.get_path(), True)
         else:
-            return (theme.mb_filetype_unknown.get_path(), True)
+            return (theme.mb_file_unknown.get_path(), True)
 
