@@ -30,6 +30,9 @@ class RootDevice(Device):
         # list of favorited files
         self.__favorites = []
         
+        # a small cache for faster access
+        self.__cache = []
+        
         # table: type -> list of devices
         self.__device_lists = {
             Device.TYPE_SYSTEM:  [],
@@ -131,9 +134,13 @@ class RootDevice(Device):
         items = []
     
         if (path.path == "/"):
-            items += self.__list_categories()
-            items += self.__list_devices(Device.TYPE_SYSTEM)
-            items += self.__list_favorites()
+            if (self.__cache):
+                items += self.__cache
+            else:
+                items += self.__list_categories()
+                items += self.__list_devices(Device.TYPE_SYSTEM)
+                items += self.__list_favorites()
+                self.__cache = items
 
         elif (path.path == "/audio"):
             items += self.__list_devices(Device.TYPE_AUDIO)
@@ -146,7 +153,6 @@ class RootDevice(Device):
 
         elif (path.path == "/storage"):
             items += self.__list_devices(Device.TYPE_GENERIC)
-
         
         cnt = 0
         for f in items:
@@ -163,6 +169,11 @@ class RootDevice(Device):
         self.emit_message(msgs.PLAYLIST_ACT_APPEND, "", f)
 
 
+    def __on_add_to_shelf(self, folder, f):
+    
+        f.bookmarked = True
+
+
     def __on_remove_from_shelf(self, folder, f):
     
         logging.debug("removing from shelf: %s", f.name)
@@ -173,6 +184,12 @@ class RootDevice(Device):
     def get_file_actions(self, folder, f):
     
         options = []
+        # allow putting storage devices on the shelf
+        print f.path
+        if (f.path == "/" and not f.bookmarked and folder.path != "/"):
+            options.append((None, "Add to Shelf",
+                           self.__on_add_to_shelf))
+
         if (f in self.__favorites):
             if (not f.full_path.startswith("playlist://")):
                 options.append((None, "Add to Playlist",
@@ -198,6 +215,7 @@ class RootDevice(Device):
 
             folder = self.get_file(_TYPE_MAP[device.TYPE])
             print "invalidating", folder, "due to", device
+            self.__cache = []
             self.emit_message(msgs.CORE_EV_FOLDER_INVALIDATED, folder)
         #end if
 
@@ -212,5 +230,6 @@ class RootDevice(Device):
                 self.__device_lists[device.TYPE].remove(device)
 
             folder = self.get_file(_TYPE_MAP[device.TYPE])
+            self.__cache = []
             self.emit_message(msgs.CORE_EV_FOLDER_INVALIDATED, folder)
         #end if
