@@ -1,6 +1,8 @@
+from com import msgs
 from storage import Device, File
 from io import SeekableFD
 from io import Downloader
+from ui.dialog import FileDialog
 from utils.MiniXML import MiniXML
 from upnp.SOAPProxy import SOAPProxy
 from upnp import didl_lite
@@ -119,7 +121,7 @@ class AVDevice(Device):
             f.name = self.get_name()
             f.icon = self.get_icon()
             f.info = "UPnP network storage"
-            f.folder_flags = f.ITEMS_SKIPPABLE | f.ITEMS_ENQUEUEABLE
+            f.folder_flags = f.ITEMS_ENQUEUEABLE
             
         else:
             if (path.startswith("/")): path = path[1:]
@@ -140,7 +142,7 @@ class AVDevice(Device):
 
             if (f.mimetype == f.DIRECTORY):
                 f.resource = ident
-                f.folder_flags = f.ITEMS_SKIPPABLE | f.ITEMS_ENQUEUEABLE
+                f.folder_flags = f.ITEMS_ENQUEUEABLE
             else:
                 f.resource = urlparse.urljoin(url_base, res)
             f.child_count = child_count
@@ -161,7 +163,7 @@ class AVDevice(Device):
 
         if (f.mimetype == f.DIRECTORY):
             f.path = "/" + ident
-            f.folder_flags = f.ITEMS_ENQUEUEABLE | f.ITEMS_SKIPPABLE
+            f.folder_flags = f.ITEMS_ENQUEUEABLE
         else:
             f.path = "/" + ident
 
@@ -177,7 +179,7 @@ class AVDevice(Device):
                               mimetypes.get_video_types() +
                               mimetypes.get_image_types() + [f.DIRECTORY]):
             ext = os.path.splitext(f.name)[-1]
-            f.mimetype = mimetypes.lookup_ext(ext)
+            f.mimetype = mimetypes.ext_to_mimetype(ext)
 
         if (f.mimetype.startswith("image/")):
             f.thumbnail = f.resource
@@ -216,24 +218,30 @@ class AVDevice(Device):
                 pass
             
         return didl
-
-
-    def __on_add_to_playlist(self, folder, f):
-    
-        self.emit_message(msgs.PLAYLIST_ACT_APPEND, "", f)
-
-
-    def __on_put_on_shelf(self, folder, f):
         
-        f.bookmarked = True
+
+    def __on_download(self, folder, f):
+
+        dlg = FileDialog(FileDialog.TYPE_SAVE, "Save File")
+        if (os.path.splitext(f.name)[1]):
+            ext = ""
+        else:
+            ext = os.path.splitext(f.resource)[1]
+        dlg.set_filename(f.name + ext)
+        dlg.run()
+        
+        destination = dlg.get_filename()
+        if (destination):
+            self.call_service(msgs.DOWNLOADER_SVC_GET, f.resource,
+                              destination)
 
 
     def get_file_actions(self, folder, f):
-    
-        actions = []
-        actions.append((None, "Add to Playlist", self.__on_add_to_playlist))
-        actions.append((None, "Put on Shelf", self.__on_put_on_shelf))
-        
+
+        actions = Device.get_file_actions(self, folder, f)
+        if (f.mimetype != f.DIRECTORY):
+            actions.append((None, "Download", self.__on_download))
+
         return actions
      
    
