@@ -209,7 +209,7 @@ class VideoPlayer(Player):
                 self.__is_playing = False
                 self.call_service(msgs.VIDEOPLAYER_SVC_RELEASE_DSP)
                 self.emit_message(msgs.MEDIA_EV_EOF)
-
+                self.emit_message(msgs.MEDIA_ACT_NEXT)
 
 
     def __on_update_position(self, ctx_id, pos, total):
@@ -219,14 +219,24 @@ class VideoPlayer(Player):
             self.__progress.set_message("%s / %s" \
                                         % (self.seconds_to_hms(pos),
                                            self.seconds_to_hms(total)))
+            self.emit_message(msgs.MEDIA_EV_POSITION, pos, total)
+
 
     def __on_change_volume(self, v):
     
         if (self.__player):
             vol = int(v * 100)
             self.__player.set_volume(vol)
+            self.__volume = vol
 
 
+    def __on_change_player_volume(self, v):
+    
+        if (v != self.__volume):
+            self.__volume_slider.set_value(v / 100.0)
+            self.__volume = v
+            
+
     def __on_seek(self, progress):
     
         if (self.__player):
@@ -260,16 +270,6 @@ class VideoPlayer(Player):
         self.emit_message(msgs.UI_ACT_FULLSCREEN, self.__is_fullscreen)
         self.__update_layout()
         self.render()
-
-
-    """
-    def _visibility_changed(self):
-    
-        if (self.is_visible()):
-            self.emit_message(msgs.ASR_ACT_FORCE_LANDSCAPE, True)
-        else:
-            self.emit_message(msgs.ASR_ACT_FORCE_LANDSCAPE, False)
-    """
             
 
     def render_this(self):
@@ -316,6 +316,7 @@ class VideoPlayer(Player):
         self.__player = self.call_service(msgs.MEDIA_SVC_GET_OUTPUT)
         self.__player.connect_status_changed(self.__on_status_changed)
         self.__player.connect_position_changed(self.__on_update_position)
+        self.__player.connect_volume_changed(self.__on_change_player_volume)
         
         self.__player.set_window(self.__screen.get_xid())
         
