@@ -2,8 +2,8 @@ import os
 import time
 
 NAME = "MediaBox"
-OSSO_NAME = "de.pycage.mediabox"
-VERSION = "2010.05.08"
+OSSO_NAME = "de.pycage.maemo.mediabox"
+VERSION = "2010.06.13"
 
 AUTHORS = ["Martin Grimme  <martin.grimme@lintegra.de>"]
 COPYRIGHT = "\xc2\xa9 2007 - 2010 Martin Grimme"
