@@ -1,5 +1,6 @@
 from AbstractBackend import AbstractBackend
 import platforms
+from utils import logging
 
 import gobject
 import pygst; pygst.require("0.10")
@@ -26,7 +27,7 @@ class GstBackend(AbstractBackend):
         # time when MediaBox has last changed the sound volume
         self.__last_volume_change_time = 0
         
-        self.__volume = 0
+        self.__volume = 50
         
         # set up device volume listener on the N900
         if (platforms.MAEMO5):
@@ -50,14 +51,19 @@ class GstBackend(AbstractBackend):
 
     def __get_current_volume(self):
     
-        try:
+        if (platforms.MAEMO5):
             bus = platforms.get_session_bus()
             obj = bus.get_object("com.nokia.mafw.renderer.Mafw-Gst-Renderer-Plugin.gstrenderer",
                                  "/com/nokia/mafw/renderer/gstrenderer")
             mafw = dbus.Interface(obj, "com.nokia.mafw.extension")
-            return int(mafw.get_extension_property("volume")[1])
-        except:
-            return 0
+            volume = int(mafw.get_extension_property("volume")[1])
+        else:
+            if (self.__player):
+                volume = self.__player.get_property("volume") * 100
+            else:
+                volume = self.__volume
+        
+        return volume
 
 
     def __load_pls(self, uri):
@@ -91,7 +97,13 @@ class GstBackend(AbstractBackend):
 
     def __start_gst(self):
 
-        self.__player = gst.element_factory_make("playbin2", "player")
+        if (platforms.MAEMO5):
+            # normal playbin is broken on Maemo5
+            self.__player = gst.element_factory_make("playbin2", "player")
+        else:
+            # playbin2 is considered experimental elsewhere
+            self.__player = gst.element_factory_make("playbin", "player")
+            
         bus = self.__player.get_bus()
         bus.add_signal_watch()
         bus.enable_sync_message_emission()
@@ -157,6 +169,9 @@ class GstBackend(AbstractBackend):
 
 
     def __on_change_volume(self, key, value):
+        """
+        Reacts on changing the system volume on the N900.
+        """
     
         # only accept system volume change notifications if the last app
         # volume change has been some time ago
@@ -185,35 +200,36 @@ class GstBackend(AbstractBackend):
             print "LOAD", uri
             self.__player.set_property("uri", uri)
             self.__player.set_state(gst.STATE_PLAYING)
-            #self.__player.set_property("volume", 1.0)
             self.__player.seek_simple(gst.Format(gst.FORMAT_TIME),
                                       gst.SEEK_FLAG_FLUSH,
                                       0)
         self._report_aspect_ratio(16/9.0)
+        if (not platforms.MAEMO5):
+            self._set_volume(self.__volume)
         self._report_volume(self.__volume)
         
 
 
     def _set_volume(self, volume):
 
-        # ugly as hell... but does its job
-        os.system("dbus-send --session --print-reply " \
-                  "--dest=com.nokia.mafw.renderer.Mafw-Gst-Renderer-Plugin.gstrenderer " \
-                  "/com/nokia/mafw/renderer/gstrenderer " \
-                  "com.nokia.mafw.extension.set_extension_property " \
-                  "string:'volume' variant:uint32:%d &" \
-                  % volume)
+        if (volume == self.__volume): return
+
+        if (platforms.MAEMO5):
+            # ugly as hell... but does its job
+            print "SET SYSTEM VOLUME", volume
+            os.system("dbus-send --session --print-reply " \
+                      "--dest=com.nokia.mafw.renderer.Mafw-Gst-Renderer-Plugin.gstrenderer " \
+                      "/com/nokia/mafw/renderer/gstrenderer " \
+                      "com.nokia.mafw.extension.set_extension_property " \
+                      "string:'volume' variant:uint32:%d 2>/dev/null &" \
+                      % volume)
+        else:
+            if (self.__player):
+                self.__player.set_property("volume", volume / 100.0)
+
         self._report_volume(volume)
         self.__last_volume_change_time = time.time()
-
-        #import dbus
-        #import platforms
-        
-        #bus = platforms.get_session_bus()
-        #obj = bus.get_object("com.nokia.mafw.renderer.Mafw-Gst-Renderer-Plugin.gstrenderer", "/com/nokia/mafw/renderer/gstrenderer")
-        #mafw_renderer = dbus.Interface(obj, "com.nokia.mafw.extension")
-        #mafw_renderer.set_extension_property("volume", volume)
-        #self.__player.set_property("volume", volume / 1000.0)
+        self.__volume = volume
 
     
     def _is_eof(self):
