@@ -1,12 +1,15 @@
 from Widget import Widget
 from ui.Pixmap import Pixmap
 from ui import windowflags
+from ui.TitleBar import TitleBar
 import platforms
+from theme import theme
 
 try:
     import gtk
 except:
     gtk = None
+import gobject
 
 try:
     import hildon
@@ -17,7 +20,8 @@ except:
 class Window(Widget):
 
     TYPE_TOPLEVEL = 0
-    TYPE_DIALOG = 1
+    TYPE_SUBWINDOW = 1
+    TYPE_DIALOG = 2
 
     RETURN_OK = 0
     RETURN_CANCEL = 1
@@ -25,13 +29,23 @@ class Window(Widget):
     RETURN_NO = 3
 
 
+    EVENT_RAISED = "event-raised"
+    EVENT_HID = "event-hid"
     EVENT_CLOSED = "event-closed"
-    
+
+    # one window may be exclusive at a time. input in other windows will be
+    # ignored and the exclusive window gets hidden in such case (e.g. for
+    # simulating the Maemo5 dialog paradigma on Maemo4)
+    __exclusive_window = [None]
+        
 
     def __init__(self, wtype):
 
         self.__wtype = wtype
         self.__flags = 0
+        
+        # widget of the windows contents
+        self.__contents = []
 
         # window menu
         if (platforms.MAEMO5):
@@ -59,16 +73,60 @@ class Window(Widget):
                 self.__window = hildon.Window()
                 self.__window.set_menu(self.__menu)
                 self.__window.fullscreen()
+            elif (platforms.MEEGO_NETBOOK):
+                self.__window = gtk.Window(gtk.WINDOW_TOPLEVEL)
+                self.__window.set_decorated(False)
+                self.__window.set_size_request(gtk.gdk.screen_width(),
+                                               gtk.gdk.screen_height())
             else:
                 self.__window = gtk.Window(gtk.WINDOW_TOPLEVEL)
+
+        elif (wtype == self.TYPE_SUBWINDOW):
+            if (platforms.MAEMO5):
+                self.__window = gtk.Dialog()                
+            elif (platforms.MAEMO4):
+                self.__window = gtk.Dialog()
+                self.__window.set_decorated(False)
+                # hide some ugly separator :)
+                self.__window.vbox.get_children()[0].hide()
+            elif (platforms.MEEGO_NETBOOK):
+                self.__window = gtk.Dialog()
+                self.__window.set_decorated(False)
+                self.__window.set_size_request(gtk.gdk.screen_width(),
+                                               gtk.gdk.screen_height())
+                # hide some ugly separator :)
+                self.__window.vbox.get_children()[0].hide()                
+            else:
+                self.__window = gtk.Dialog()
+                # hide some ugly separator :)
+                self.__window.vbox.get_children()[0].hide()
+
                             
         elif (wtype == self.TYPE_DIALOG):
-            if (platforms.MAEMO4):
-                self.__window = hildon.Window()
-                self.__window.fullscreen()
+            if (platforms.MAEMO5):
+                self.__window = gtk.Dialog()
+            elif (platforms.MAEMO4 or platforms.MEEGO_NETBOOK):
+                self.__window = gtk.Dialog()
+                # hide some ugly separator :)
+                self.__window.vbox.get_children()[0].hide()
             else:
                 self.__window = gtk.Dialog()
 
+        # title bar on some platforms
+        if (not platforms.MAEMO5 and wtype != self.TYPE_DIALOG):
+            self.__title_bar = TitleBar()
+            self.__title_bar.connect_switch(lambda :self.__window.iconify())
+            self.__title_bar.connect_menu(lambda :self.show_menu())
+            self.__title_bar.connect_close(
+                                  lambda :self.emit_event(self.EVENT_CLOSED))
+            if (wtype == self.TYPE_SUBWINDOW):
+                self.__title_bar.set_mode(TitleBar.MODE_SUBWINDOW)
+
+            Widget.add(self, self.__title_bar)
+
+        else:
+            self.__title_bar = None
+            
         self.__window.set_title(" ")
         self.__window.set_default_size(800, 480)
         self.__window.set_app_paintable(True)
@@ -93,14 +151,17 @@ class Window(Widget):
         self.__window.realize()
 
         if (platforms.MAEMO5):
-            self.__set_portrait_property("_HILDON_PORTRAIT_MODE_SUPPORT", 1)
+            if (wtype == self.TYPE_TOPLEVEL):
+                self.__set_portrait_property("_HILDON_PORTRAIT_MODE_SUPPORT", 1)
+            else:
+                self.__set_portrait_property("_HILDON_PORTRAIT_MODE_SUPPORT", 1)
 
         self.__layout = gtk.Fixed()
         self.__layout.show()
-        if (wtype == self.TYPE_TOPLEVEL or platforms.MAEMO4):
-            self.__window.add(self.__layout)
-        else:
+        try:
             self.__window.vbox.add(self.__layout)
+        except:
+            self.__window.add(self.__layout)
         
         # video screen
         self.__vidscreen = gtk.DrawingArea()
@@ -115,12 +176,24 @@ class Window(Widget):
                                     gtk.gdk.KEY_RELEASE_MASK)
         self.__layout.put(self.__vidscreen, 0, 0)
 
+        gobject.timeout_add(0, self.__update_window_background)
+
 
     def connect_closed(self, cb, *args):
     
         self._connect(self.EVENT_CLOSED, cb, *args)
 
 
+    def connect_raised(self, cb, *args):
+    
+        self._connect(self.EVENT_RAISED, cb, *args)
+
+
+    def connect_hid(self, cb, *args):
+    
+        self._connect(self.EVENT_HID, cb, *args)
+
+
     def get_window(self):
     
         return self
@@ -136,9 +209,12 @@ class Window(Widget):
 
         changes = []
         for flag in [windowflags.FULLSCREEN,
+                     windowflags.ASR,
                      windowflags.PORTRAIT,
                      windowflags.CATCH_VOLUME_KEYS,
-                     windowflags.BUSY]:
+                     windowflags.BUSY,
+                     windowflags.FRAMELESS,
+                     windowflags.EXCLUSIVE]:
             if (flags & flag != self.__flags &  flag):
                 changes.append((flag, flags & flag))
         #end for
@@ -166,6 +242,53 @@ class Window(Widget):
         self.set_flags(new_flags)
 
 
+    """
+    def get_size(self):
+    
+        w, h = Widget.get_size(self)
+        return (696, 396)
+
+
+    def get_screen_pos(self):
+    
+        return (12, 0)
+    """
+
+
+    def add(self, w):
+        """
+        Adds a widget to this window. If you don't override L{render_this}, all
+        widgets occupy the whole window area.
+        """
+    
+        Widget.add(self, w)
+        if (not w in self.__contents):
+            self.__contents.append(w)
+
+
+    def __check_exclusive(self):
+    
+        if (self.__exclusive_window[0] and
+            self.__exclusive_window[0] != self):
+            self.__exclusive_window[0].set_visible(False)
+            self.__exclusive_window[0] = None
+            return False
+        else:
+            return True
+
+
+    def _reload(self):
+
+        self.__update_window_background()
+        
+
+    def __update_window_background(self):
+        
+        self.__window.modify_bg(gtk.STATE_NORMAL,
+                          gtk.gdk.color_parse(str(theme.color_mb_background)))
+
+
+
     def __on_configure(self, src, ev):
 
         w, h = self.__window.get_size()
@@ -194,6 +317,9 @@ class Window(Widget):
 
     def __on_button_pressed(self, src, ev):
 
+        if (not self.__check_exclusive()):
+            return
+
         if (ev.button == 1):
             px, py = src.get_pointer()
             self._handle_event(self.EVENT_BUTTON_PRESS, px, py)
@@ -227,6 +353,9 @@ class Window(Widget):
 
 
     def __on_key_pressed(self, src, ev):
+
+        if (not self.__check_exclusive()):
+            return
     
         keyval = ev.keyval
         c = gtk.gdk.keyval_to_unicode(keyval)
@@ -267,20 +396,39 @@ class Window(Widget):
                                              [value])
 
 
+    def __unset_portrait_property(self, prop):
+
+        self.__window.window.property_delete(prop)
+
+
     def _update_flag(self, flag, value):
 
         if (flag == windowflags.FULLSCREEN):
-            if (not platforms.MAEMO4):
+            if (platforms.MAEMO4 or platforms.MEEGO_NETBOOK):
+                if (self.__title_bar):
+                    self.__title_bar.set_visible(not value)
+                    self.render()
+            else:
                 if (value):
                     self.__window.fullscreen()
                 else:
                     self.__window.unfullscreen()
+        
+        elif (flag == windowflags.ASR):
+            if (platforms.MAEMO5):
+                if (value):
+                    self.__set_portrait_property("_HILDON_PORTRAIT_MODE_SUPPORT", 1)
+                    #self.__set_portrait_property("_HILDON_PORTRAIT_MODE_REQUEST", 0)
+                else:
+                    self.__unset_portrait_property("_HILDON_PORTRAIT_MODE_SUPPORT")
 
         elif (flag == windowflags.PORTRAIT):
             if (platforms.MAEMO5):
-                self.__set_portrait_property("_HILDON_PORTRAIT_MODE_SUPPORT", 1)
-                self.__set_portrait_property("_HILDON_PORTRAIT_MODE_REQUEST",
-                                             value and 1 or 0)
+                #self.__set_portrait_property("_HILDON_PORTRAIT_MODE_SUPPORT", 1)
+                if (value):
+                    self.__set_portrait_property("_HILDON_PORTRAIT_MODE_REQUEST", 1)
+                else:
+                    self.__unset_portrait_property("_HILDON_PORTRAIT_MODE_REQUEST")
 
         elif (flag == windowflags.BUSY):
             if (platforms.MAEMO5):
@@ -294,7 +442,11 @@ class Window(Widget):
                                                      gtk.gdk.PROP_MODE_REPLACE,
                                                      [value and 1 or 0])
 
+        elif (flag == windowflags.FRAMELESS):
+            self.__title_bar.set_visible(not value)
 
+        elif (flag == windowflags.EXCLUSIVE):
+            self.__exclusive_window[0] = self
 
 
     def _get_window_impl(self):
@@ -306,25 +458,40 @@ class Window(Widget):
 
         if (self.is_visible()):
             if (platforms.MAEMO5):
-                if (self.__wtype == self.TYPE_DIALOG and
-                    not self.__has_size_set):
-                    self.__window.resize(gtk.gdk.screen_width(),
-                                         gtk.gdk.screen_height() - 120)
-            #endif MAEMO5
-            self.__window.show()
+                if (self.__wtype != self.TYPE_TOPLEVEL):
+                    if (self.__has_size_set):
+                        w, h = self.get_size()
+                        self.__window.resize(gtk.gdk.screen_width(), h)
+                    else:
+                        self.__window.resize(gtk.gdk.screen_width(),
+                                             gtk.gdk.screen_height() - 120)
+
+            elif (platforms.MAEMO4):
+                if (self.__wtype != self.TYPE_TOPLEVEL and self.__has_size_set):
+                    w, h = self.__window.get_size()
+                    self.__window.move(0, 480 - h)
+
+            #endif
+
             self.render()
+            self.__window.show()
+            self.emit_event(self.EVENT_RAISED)
         else:
+            self.emit_event(self.EVENT_HID)
             self.__window.hide()
 
 
     def set_parent_window(self, parent):
     
-        self.__window.set_transient_for(parent._get_window_impl())
+        if (not platforms.MEEGO_NETBOOK):
+            self.__window.set_transient_for(parent._get_window_impl())
 
 
     def destroy(self):
     
         self.__window.destroy()
+        if (self.__exclusive_window[0] == self):
+            self.__exclusive_window[0] = None
 
 
     def has_focus(self):
@@ -334,13 +501,39 @@ class Window(Widget):
 
     def set_window_size(self, w, h):
     
+        if (self.__title_bar):
+            h += 57
         self.__window.resize(w, h)
         self.__has_size_set = True
 
 
+    def render_this(self):
+    
+        w, h = self.get_size()
+        if (self.__title_bar and self.__title_bar.is_visible()):
+            self.__title_bar.set_geometry(0, 0, w, 57)
+            offset = 57
+        else:
+            offset = 0
+        for c in self.__contents:
+            c.set_geometry(0, offset, w, h - offset)
+
+
     def set_title(self, title):
     
         self.__window.set_title(title)
+        if (self.__title_bar):
+            self.__title_bar.set_title(title)
+
+
+    def show_menu(self):
+    
+        def positioner(menu):
+            x, y = self.__window.window.get_position()
+            return (x + 80, y + 57, False)
+
+        if (self.__menu_items):
+            self.__menu.popup(None, None, positioner, 1, 0)
 
 
     def set_menu_item(self, name, label, visible, cb):
@@ -456,3 +649,6 @@ class Window(Widget):
             gtk.main_iteration(True)
         #end while
 
+        if (self.__exclusive_window[0] == self):
+            self.__exclusive_window[0] = None
+
